/************************************************************
*
*
*	Unit containing main assembling code.
*
*	by Adrian Bool in cooperation with Graham Cox.
*
*	copyright  phantasm coding 1992.
*
*
************************************************************/

#include "Global.h"
#include "Assemble.h"

#include "Ass.proto.h"

pHandle newProgram(void)
	{
	pHandle		temp;
	
	temp = (pHandle) NewHandle(sizeof(program)); 
	
	if (temp != nil) 
		{
		(*temp)->sourceCode = newSource();
		(*temp)->objectCode = newObject();
		(*temp)->labelHeader = newLabelList();
		}
	return(temp);
	}
	
	
void disposeProgram(pHandle theProgram)
	{
	disposeSource((*theProgram)->sourceCode);
	disposeObject((*theProgram)->objectCode);
	disposeLabelList((*theProgram)->labelHeader);
	
	DisposHandle((Handle) theProgram);
	}
	
void assemble(pHandle theProgram)
	{
	passOne(theProgram);
	createObjectSpace((*theProgram)->objectCode);
	passTwo(theProgram);

	}
	
void passOne(pHandle theProgram)
	{
	str255 aSection;
	pdpTableRef theOpcode;
	int sectionOk;
	
	rBlock resultBlock;
	short resultSize;
	
	sHandle sourceCode;
	oHandle objectCode;
	lHandle labelHeader;
	
	sourceCode = (*theProgram)->sourceCode;
	objectCode = (*theProgram)->objectCode;
	labelHeader = (*theProgram)->labelHeader;
	
	(*sourceCode)->line = 0;
	
	if (sourceCode != nil && objectCode != nil && labelHeader != nil)
		{
		
		(*objectCode)->address = 0;
		(*objectCode)->position = 0;
		(*sourceCode)->line = 1;
		do
			{
			getSection(theProgram,aSection);
			if (!AsmError) 
				{
				if (aSection[0] == ';') 
					{	
					/* a comment has been encountered */
					
					nextLine(sourceCode);
					}  
				else
					{	
					if (aSection[lengthOfString(aSection)-1] == ':')
						{	
						/* a defining label has been found */
						
						newLabel(theProgram,aSection,(*objectCode)->address);					
						}
					
					else
						{	
						if (theOpcode = getOpcode(aSection))
							{	
							/* an opcode has been found */
							
							assembleOpcode(theOpcode,1,resultBlock,&resultSize,theProgram);
							(*objectCode)->position += resultSize;
							(*objectCode)->address += resultSize;
							}
						}
					}
				}
			}
		while (theOpcode != end && AsmError==noErr);
		(*(*theProgram)->objectCode)->size = (*(*theProgram)->objectCode)->position;
		}
	else
		AsmError = NotEnoughMemory;
	}
	

void passTwo(pHandle theProgram)
	{
	str255 aSection;
	pdpTableRef theOpcode;
	int sectionOk;
	
	rBlock resultBlock;
	short resultSize;
	
	sHandle sourceCode;
	oHandle objectCode;
	lHandle labelHeader;
	
	sourceCode = (*theProgram)->sourceCode;
	objectCode = (*theProgram)->objectCode;
	labelHeader = (*theProgram)->labelHeader;
	
	(*sourceCode)->line = 0;
	
	if (sourceCode != nil && objectCode != nil && labelHeader != nil)
		{
		(*objectCode)->position = 0;
		(*objectCode)->address = (*objectCode)->startAddress;
		(*sourceCode)->caret = 0;
		(*sourceCode)->line = 1;
		
		do
			{
			theOpcode = 0;
			getSection(theProgram,aSection);
			if (!AsmError) 
				{
				if (aSection[0] == ';') 
					{	
					/* a comment has been encountered */
					
					nextLine(sourceCode);
					}  
				else
					{	
					if (aSection[lengthOfString(aSection)-1] != ':')
						{	
						if (theOpcode = getOpcode(aSection))
							{	
							/* an opcode has been found */
							
							assembleOpcode(theOpcode,2,resultBlock,&resultSize,theProgram);
							
							addBlock(objectCode,resultBlock,resultSize);
							(*objectCode)->address += resultSize;
							(*objectCode)->position += resultSize;
							}
						}
					}
				}
			}
		while (theOpcode != end && AsmError == noErr);
		}
	else
		AsmError = NotEnoughMemory;
	}


